import { ObjectId } from "mongodb"
import { getUserById } from "../db/users";
import { Users } from "../db/mongodb";
import type DBResource from "../../types/DBResource";
import type Resource from "../../types/Resource";

export const createInitialResources = async (id: ObjectId) => {
    const resources: Array<DBResource> = [
        {
            name: "coal",
            amount: 11,
            lastUpdated: new Date(),
            perHourMiningRate: 11
        },
        {
            name: "iron",
            amount: 22,
            lastUpdated: new Date(),
            perHourMiningRate: 22
        },
        {
            name: "gold",
            amount: 33,
            lastUpdated: new Date(),
            perHourMiningRate: 33
        },
        {
            name: "water",
            amount: 44,
            lastUpdated: new Date(),
            perHourMiningRate: 44
        },
        {
            name: "sulfuricAcid",
            amount: 55,
            lastUpdated: new Date(),
            perHourMiningRate: 55
        },
        {
            name: "liquidNitrogen",
            amount: 66,
            lastUpdated: new Date(),
            perHourMiningRate: 66
        },
        {
            name: "hydrogen",
            amount: 77,
            lastUpdated: new Date(),
            perHourMiningRate: 77
        },
        {
            name: "oxygen",
            amount: 88,
            lastUpdated: new Date(),
            perHourMiningRate: 88
        },
        {
            name: "helium3",
            amount: 99,
            lastUpdated: new Date(),
            perHourMiningRate: 99
        }
    ];

    updateUserResources(id, resources);
}

export const getUserResources = async (id: ObjectId): Promise<Array<DBResource>> => {
    const user = await getUserById(id);

    const defaultResources: Array<DBResource> = [];

    return user?.resources === undefined ? defaultResources : user?.resources;
}

export const updateUserResources = async (id: ObjectId, resources: Array<DBResource>) => {
    const users = await Users();
    await users.updateOne({ _id: id }, {
        $set: {
            resources
        }
    });
}

export const calculateCurrentAvailableResources = async (id: ObjectId): Promise<Array<DBResource>> => {
    const user = await getUserById(id);

    if(user === null) return [];

    const userResources = await getUserResources(id);

    userResources.forEach(res => {
        const timeDiff = Math.abs((new Date()).getTime() - res.lastUpdated.getTime());
        const hours = timeDiff / (1000 * 60 * 60);
        const amountToAdd = hours * res.perHourMiningRate;
        res.amount += amountToAdd;
        res.lastUpdated = new Date();
    });

    updateUserResources(id, userResources);
    return userResources;
}