import { type APIRoute } from "astro";
import validateAccessToken from "../../../lib/utils/validateAccessToken";
import { getAccessToken } from "../../../lib/db/accessTokens";
// import { getUserResources } from "../../../lib/db/users";
import { getUserResources } from "../../../lib/utils/resourceManager";
import buildings from '../../../lib/data/buildings.json';
import calculateAvailableResources from "../../../lib/utils/calculateAvailableResources";
import { getUserByAccessToken } from "../../../lib/db/users";

export const POST: APIRoute = async({ request }) => {
    const response = await validateAccessToken(request);
    if(response instanceof Response) return response;

    const user = await getUserByAccessToken(response);
    if(user === null) {
        return new Response(
            JSON.stringify({
                code: 401,
                message: "Unauthorized"
            }), { status: 401 }
        )
    }
    const resources = await getUserResources(user._id);
    const buildingId = (await request.json()).building;
    const building = buildings.map(cat => cat.buildings.filter(b => b.id === buildingId))[0][0];
    const balance = calculateAvailableResources(resources, building.requirements.resources);
    
    return new Response(
        JSON.stringify({
            code: 200,
            message: "OK",
            accessToken: response,
            data: {
                resources,
                building,
                balance
            }
        }), { status: 200 }
    )
}